import java.awt.*;
import java.awt.geom.*;
import java.lang.Math.*;

/**
 * Implementacion de la figura geometrica Rectangulo
 * @author Carlos Cortes
 */
public class Rectangulo extends Forma
{
    /**
     * Ubicacion del punto central
     */
    private Point2D.Double posicion;
    
    /**
     * Inclinacion en radianes
     */
    private double inclin;
    
    /**
     * Longitud horizontal cuando la inclinacion es cero
     */
    private double ancho;
    
    /**
     * Longitud vertical cuando la inclinacion es cero
     */
    private double alto;
    
    /**
     * Constructor
     * Crea e inicializa un nuevo Rectangulo definido por dos puntos
     * @param p1,p2 Dos puntos que definen el rectangulo
     */
    public Rectangulo(Point p1, Point p2)
    {
        this.ancho    = p2.getX() - p1.getX();
        this.alto     = p2.getY() - p1.getY();
        if (this.ancho < 0) this.ancho *= -1;
        if (this.alto  < 0) this.alto  *= -1;
        
        this.posicion = new Point2D.Double((double)(p2.getX()+p1.getX())/2,
                                           (double)(p2.getY()+p1.getY())/2);
        this.inclin   = 0;
    }
    
    /**
     * Retorna la posicion del rectangulo
     */
    public Point2D.Double getPosicion()
    {
        return this.posicion;
    }
    
    /**
     * Retorna la inclinacion en radianes del rectangulo
     */
    public double getInclin()
    {
        return this.inclin;
    }
    
    /**
     * Retorna el ancho del rectangulo
     */
    public double getAncho()
    {
        return this.ancho;
    }
    
    /**
     * Retorna el alto del rectangulo
     */
    public double getAlto()
    {
        return this.alto;
    }
    
    /**
     * Establece la posicion del rectangulo
     */
    public void setPosicion(Point2D.Double newPos)
    {
        this.posicion = newPos;
        return;
    }
    
    /**
     * Normaliza y establece la inclinacion en radianes del rectangulo
     */
    public void setInclin(double newInclin)
    {
        double temp;
        if (newInclin >= 0)
            if ( Math.floor(newInclin/Math.PI) % 2 == 0 )
                newInclin %= Math.PI;
            else
                newInclin = -Math.PI + newInclin % Math.PI;
        else
            if ( (Math.floor(-newInclin/Math.PI)) % 2 == 0 )
                newInclin = -(-newInclin % Math.PI);
            else
                newInclin = -(-Math.PI + (-newInclin % Math.PI));
        
        this.inclin = newInclin;
        return;
    }
    
    /**
     * Establece el ancho del rectangulo
     */
    public void setAncho(double newAncho)
    {
        this.ancho = newAncho;
        return;
    }
    
    /**
     * Establece el alto del rectangulo
     */
    public void setAlto(double newAlto)
    {
        this.alto = newAlto;
        return;
    }
    
    public void escalar(Point centro, double factor)
    {
        double fx, fy;
        
        fx = this.posicion.getX() - centro.getX();
        fy = this.posicion.getY() - centro.getY();
        fx *= factor;
        fy *= factor;
        fx += centro.getX();
        fy += centro.getY();
        
        this.posicion.setLocation(fx, fy);
        
        this.ancho *= factor;
        this.alto  *= factor;
        
        return;
    }
    
    public void rotar(Point centro, double deltaAngulo)
    {
        double fx, fy,  // vector posicion antes del giro
               rx, ry,  // vetor auxiliar para el calculo
               gx, gy;  // vector posicion despues del giro
        
        this.setInclin(inclin + deltaAngulo);
        
        // Se hace que 'centro' sea el origen de coordenadas
        fx = this.posicion.getX() - centro.getX();
        fy = this.posicion.getY() - centro.getY();
        
        // Se calcula la componente X del nuevo vector posicion
        deltaAngulo *= -1;
        rx = Math.cos(deltaAngulo);
        ry = Math.sin(deltaAngulo);
        gx = fx*rx + fy*ry; // 'proyeccion' de 'f' sobre 'r'
        
        // Se calcula la componente Y del nuevo vetor posicion
        deltaAngulo += Math.PI/2;
        rx = Math.cos(deltaAngulo);
        ry = Math.sin(deltaAngulo);
        gy = fx*rx + fy*ry; // 'proyeccion' de 'f' sobre 'r'
        
        // Se traslada 'centro' a su posicion original
        gx += centro.getX();
        gy += centro.getY();
        this.posicion.setLocation(gx, gy);
        
        return;
    }
    
    public void trasladar(double deltaX, double deltaY)
    {
        this.posicion.setLocation(this.posicion.getX() + deltaX,
                                  this.posicion.getY() + deltaY);
        return;
    }
    
    /**
     * Retorna un String que describe la informacion mas relevante del Rectangulo.
     * Se limita la presicion a dos decimales (por redondeo).
     */
    public String toString()
    {
        return "Este es un Rectangulo: Centro=( "
               + Math.rint(this.posicion.getX()*100)/100.0
               + " , "
               + Math.rint(this.posicion.getY()*100)/100.0
               + " ) ; Ancho="
               + Math.rint(this.ancho*100)/100.0
               + "; alto="
               + Math.rint(this.alto*100)/100.0
               + ", inclinacion="
               + Math.rint(Math.toDegrees(this.inclin) *100)/100.0
               + " grados.";
    }
    
    /**
     * Indica si el rectangulo (this) es igual a otro
     */
    public boolean equals(Rectangulo otro)
    {
        if (this == otro) return true;
        if (otro == null) return false;
        if (this.getClass() != otro.getClass()) return false;
        
        if (this.posicion.equals(otro.getPosicion()) && 
            this.inclin == otro.getInclin() &&
            this.ancho == otro.getInclin() &&
            this.alto == otro.getAlto()
            ) return true;
        else
            return false;
    }
    
    /**
     * Crea otro rectangulo identico
     */
    public Rectangulo clone()
    {
        Rectangulo hijo = new Rectangulo(new Point(), new Point());
        hijo.setPosicion(this.getPosicion());
        hijo.setInclin(this.getInclin());
        hijo.setAncho(this.getAncho());
        hijo.setAlto(this.getAlto());
        return hijo;
    }
}